(* $Id: mapred_sort.ml 345 2011-01-14 20:02:11Z gerd $ *)

(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file ../LICENSE      *)
(*  of the Objective Caml distribution.                                *)
(*                                                                     *)
(***********************************************************************)

(* A slightly optimized version of Array.stable_sort. This is
   copyright INRIA (see copyright notice in array.ml)
 *)

let cutoff = 10;;
let merge_sort cmp (a : int array) =
  let merge src1ofs src1len src2 src2ofs src2len dst dstofs =
    let src1r = src1ofs + src1len and src2r = src2ofs + src2len in
    let rec loop i1 s1 i2 s2 d =
      if cmp s1 s2 <= 0 then begin
        Array.unsafe_set dst d s1;
        let i1 = i1 + 1 in
        if i1 < src1r then
          loop i1 (Array.unsafe_get a i1) i2 s2 (d + 1)
        else
          Array.blit src2 i2 dst (d + 1) (src2r - i2)
      end else begin
        Array.unsafe_set dst d s2;
        let i2 = i2 + 1 in
        if i2 < src2r then
          loop i1 s1 i2 (Array.unsafe_get src2 i2) (d + 1)
        else
          Array.blit a i1 dst (d + 1) (src1r - i1)
      end
    in 
    loop 
      src1ofs (Array.unsafe_get a src1ofs) 
      src2ofs (Array.unsafe_get src2 src2ofs) 
      dstofs;
  in
  let isortto srcofs dst dstofs len =
    for i = 0 to len - 1 do
      let e = (Array.unsafe_get a (srcofs + i)) in
      let j = ref (dstofs + i - 1) in
      while (!j >= dstofs && cmp (Array.unsafe_get dst !j) e > 0) do
        Array.unsafe_set dst (!j + 1) (Array.unsafe_get dst !j);
        decr j;
      done;
      Array.unsafe_set dst (!j + 1) e;
    done;
  in
  let rec sortto srcofs dst dstofs len =
    if len <= cutoff then isortto srcofs dst dstofs len else begin
      let l1 = len / 2 in
      let l2 = len - l1 in
      sortto (srcofs + l1) dst (dstofs + l1) l2;
      sortto srcofs a (srcofs + l2) l1;
      merge (srcofs + l2) l1 dst (dstofs + l1) l2 dst dstofs;
    end;
  in
  let l = Array.length a in
  if l <= cutoff then isortto 0 a 0 l else begin
    let l1 = l / 2 in
    let l2 = l - l1 in
    let t = Array.make l2 (Array.unsafe_get a 0) in
    sortto l1 t 0 l2;
    sortto 0 a l2 l1;
    merge l2 l1 t 0 l2 a 0;
  end;
;;


let ba_merge_sort
      cmp (a : (int64,Bigarray.int64_elt,Bigarray.c_layout) Bigarray.Array1.t)=
  let open Bigarray.Array1 in
  let merge src1ofs src1len src2 src2ofs src2len dst dstofs =
    let src1r = src1ofs + src1len and src2r = src2ofs + src2len in
    let rec loop i1 s1 i2 s2 d =
      if cmp s1 s2 <= 0 then begin
        unsafe_set dst d s1;
        let i1 = i1 + 1 in
        if i1 < src1r then
          loop i1 (unsafe_get a i1) i2 s2 (d + 1)
        else
	  let l = src2r - i2 in
	  blit (sub src2 i2 l) (sub dst (d+1) l)
      end else begin
        unsafe_set dst d s2;
        let i2 = i2 + 1 in
        if i2 < src2r then
          loop i1 s1 i2 (unsafe_get src2 i2) (d + 1)
        else
	  let l = src1r - i1 in
	  blit (sub a i1 l) (sub dst (d+1) l)
      end
    in 
    loop 
      src1ofs (unsafe_get a src1ofs) 
      src2ofs (unsafe_get src2 src2ofs) 
      dstofs;
  in
  let isortto srcofs dst dstofs len =
    for i = 0 to len - 1 do
      let e = (unsafe_get a (srcofs + i)) in
      let j = ref (dstofs + i - 1) in
      while (!j >= dstofs && cmp (unsafe_get dst !j) e > 0) do
        unsafe_set dst (!j + 1) (unsafe_get dst !j);
        decr j;
      done;
      unsafe_set dst (!j + 1) e;
    done;
  in
  let rec sortto srcofs dst dstofs len =
    if len <= cutoff then isortto srcofs dst dstofs len else begin
      let l1 = len / 2 in
      let l2 = len - l1 in
      sortto (srcofs + l1) dst (dstofs + l1) l2;
      sortto srcofs a (srcofs + l2) l1;
      merge (srcofs + l2) l1 dst (dstofs + l1) l2 dst dstofs;
    end;
  in
  let l = dim a in
  if l <= cutoff then isortto 0 a 0 l else begin
    let l1 = l / 2 in
    let l2 = l - l1 in
    let t = create Bigarray.int64 Bigarray.c_layout l2 in
    sortto l1 t 0 l2;
    sortto 0 a l2 l1;
    merge l2 l1 t 0 l2 a 0;
  end;
;;
