(*
  Copyright 2010 Gerd Stolpmann

  This file is part of Plasma, a distributed filesystem and a
  map/reduce computation framework. Unless you have a written license
  agreement with the copyright holder (Gerd Stolpmann), the following
  terms apply:

  Plasma is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Plasma is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

*)
(* $Id: platform.ml 359 2011-01-31 15:46:09Z gerd $ *)

external find_lf : Netsys_mem.memory -> int -> int -> int 
  = "mapred_find_lf"

external rev_find_lf : Netsys_mem.memory -> int -> int -> int 
  = "mapred_rev_find_lf"

let get_number_from_script script =
  try
    let ch = Unix.open_process_in script in
    let ch_open = ref true in
    try
      let n = input_line ch in
      let st = Unix.close_process_in ch in
      ch_open := false;
      if st <> Unix.WEXITED 0 then raise Not_found;
      Some(Int64.of_float(float_of_string n))
    with
      | error ->
	  if !ch_open then
	    ignore(Unix.close_process_in ch);
	  None
  with
    | error ->
	None


let physical_ram_script = include_file "physical_ram.sh"

let physical_ram() =
  get_number_from_script physical_ram_script

let number_cores_script = include_file "number_cores.sh"

let number_cores() =
  match get_number_from_script number_cores_script with
    | None -> None
    | Some n -> Some(Int64.to_int n)

let shm_limit() =
  match physical_ram() with
    | None -> None
    | Some n -> Some(Int64.div n 8L)
