/* $Id: platform_c.c 275 2010-10-25 01:30:28Z gerd $ */

/* Helpers for mapred performance */

#define _GNU_SOURCE 

#include <string.h>
#include <caml/fail.h>
#include <caml/bigarray.h>

/* external find_lf :  Netsys_mem.memory -> int -> int -> int 
  = "mapred_find_lf"

   Looks for LF char from mem[pos] to mem[endpos-1] and returns the
   position. Not_found if not found.
 */

CAMLprim value mapred_find_lf (value memv, value posv, value endposv) {
    struct caml_bigarray *mem = Bigarray_val(memv);
    long pos = Long_val(posv);
    long endpos = Long_val(endposv);
    char *m = (char *) mem->data;
    char *found;

    found = (char *) memchr(m + pos, '\n', endpos-pos);
    if (found == NULL) caml_raise_not_found();

    return (Val_long (found - m));
}


#ifdef __linux__
#define HAVE_MEMRCHR
#endif

#ifdef __FreeBSD__
#define HAVE_MEMRCHR
#endif

#ifdef __OpenBSD__
#define HAVE_MEMRCHR
#endif

CAMLprim value mapred_rev_find_lf (value memv, value posv, value endposv) {
    struct caml_bigarray *mem = Bigarray_val(memv);
    long pos = Long_val(posv);
    long endpos = Long_val(endposv);
    char *m = (char *) mem->data;
    char *found;

#ifdef HAVE_MEMRCHR    
    found = (char *) memrchr(m + pos, '\n', endpos-pos);
#else
    found = NULL;
    while (endpos > pos && m[endpos-1] != '\n') endpos--;
    if (endpos > pos) found = m+endpos;
#endif
    if (found == NULL) caml_raise_not_found();

    return (Val_long (found - m));
}
